@@ -31,14 +31,16 @@
 #include "fr-stock.h"
 #include "main.h"
 #include "gtk-utils.h"
-#include "window.h"
+#include "fr-window.h"
 #include "typedefs.h"
 #include "gconf-utils.h"
 
 
 typedef struct {
-	FRWindow     *window;
-
+	FrWindow     *window;
+	GList        *selected_files;
+	char         *base_dir_for_selection;
+	
 	GtkWidget    *dialog;
 
 	GtkWidget    *e_main_vbox;
@@ -66,10 +68,11 @@ destroy_cb (GtkWidget  *widget,
 	    DialogData *data)
 {
 	if (! data->extract_clicked) {
-		window_pop_message (data->window);
-		window_batch_mode_stop (data->window);
+		fr_window_pop_message (data->window);
+		fr_window_stop_batch (data->window);
 	}
-
+	path_list_free (data->selected_files);
+	g_free (data->base_dir_for_selection);
 	g_object_unref (data->tooltips);
 	g_free (data);
 }
@@ -79,7 +82,7 @@ static int
 extract_cb (GtkWidget   *w,
 	    DialogData  *data)
 {
-	FRWindow   *window = data->window;
+	FrWindow   *window = data->window;
 	gboolean    do_not_extract = FALSE;
 	char       *extract_to_dir;
 	gboolean    overwrite;
@@ -89,18 +92,18 @@ extract_cb (GtkWidget   *w,
 	gboolean    junk_paths;
 	GList      *file_list;
 	char       *password;
-	const char *base_dir = NULL;
+	char       *base_dir = NULL;
 
 	data->extract_clicked = TRUE;
 
 	/* collect extraction options. */
 
-	extract_to_dir = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (data->dialog));
+	extract_to_dir = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (data->dialog));
 
 	/* check directory existence. */
 
 	if (! path_is_dir (extract_to_dir)) {
-		if (! force_directory_creation) {
+		if (! ForceDirectoryCreation) {
 			GtkWidget *d;
 			int        r;
 			char      *folder_name;
@@ -131,24 +134,20 @@ extract_cb (GtkWidget   *w,
 
 		if (! do_not_extract && ! ensure_dir_exists (extract_to_dir, 0755)) {
 			GtkWidget  *d;
-			const char *error;
 			char       *message;
 
-			error = gnome_vfs_result_to_string (gnome_vfs_result_from_errno ());
-			message = g_strdup_printf (_("Could not create the destination folder: %s."), error);
-			d = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-						     GTK_DIALOG_DESTROY_WITH_PARENT,
-						     GTK_STOCK_DIALOG_ERROR,
-						     _("Extraction not performed"),
-						     message,
-						     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
-						     NULL);
-			gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
-			g_free (message);
+			message = g_strdup_printf (_("Could not create the destination folder: %s."), gnome_vfs_result_to_string (gnome_vfs_result_from_errno ()));
 
+			d = _gtk_error_dialog_new (GTK_WINDOW (window),
+						   GTK_DIALOG_DESTROY_WITH_PARENT,
+						   NULL,
+						   _("Extraction not performed"),
+						   message);
 			gtk_dialog_run (GTK_DIALOG (d));
 			gtk_widget_destroy (GTK_WIDGET (d));
 
+			g_free (message);
+
 			return FALSE;
 		}
 	}
@@ -156,18 +155,18 @@ extract_cb (GtkWidget   *w,
 	if (do_not_extract) {
 		GtkWidget *d;
 
-		d = _gtk_message_dialog_new (GTK_WINDOW (window->app),
+		d = _gtk_message_dialog_new (GTK_WINDOW (window),
 					     GTK_DIALOG_DESTROY_WITH_PARENT,
-					     GTK_STOCK_DIALOG_ERROR,
+					     GTK_STOCK_DIALOG_WARNING,
 					     _("Extraction not performed"),
 					     NULL,
-					     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
+					     GTK_STOCK_OK, GTK_RESPONSE_OK,
 					     NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
+		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
 
-		if (data->window->batch_mode)
+		if (fr_window_is_batch_mode (data->window))
 			gtk_widget_destroy (data->dialog);
 
 		return FALSE;
@@ -179,31 +178,25 @@ extract_cb (GtkWidget   *w,
 	    && ! check_permissions (extract_to_dir, R_OK | W_OK | X_OK)) {
 		GtkWidget *d;
 		char      *utf8_path;
-		char      *message;
 
 		utf8_path = g_filename_display_name (extract_to_dir);
-		message = g_strdup_printf (_("You don't have the right permissions to extract archives in the folder \"%s\""), utf8_path);
-		g_free (utf8_path);
-		d = _gtk_message_dialog_new (GTK_WINDOW (window->app),
-					     GTK_DIALOG_DESTROY_WITH_PARENT,
-					     GTK_STOCK_DIALOG_ERROR,
-					     _("Extraction not performed"),
-					     message,
-					     GTK_STOCK_OK, GTK_RESPONSE_CANCEL,
-					     NULL);
-		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
-		g_free (message);
+
+		d = _gtk_error_dialog_new (GTK_WINDOW (window),
+					   GTK_DIALOG_DESTROY_WITH_PARENT,
+					   NULL,
+					   _("Extraction not performed"),
+					   _("You don't have the right permissions to extract archives in the folder \"%s\""), 
+					   utf8_path);
 		gtk_dialog_run (GTK_DIALOG (d));
 		gtk_widget_destroy (GTK_WIDGET (d));
 
+		g_free (utf8_path);
 		g_free (extract_to_dir);
 
 		return FALSE;
 	}
 
-	window_set_extract_default_dir (window, extract_to_dir);
-	if (window->batch_mode)
-		window->extract_interact_use_default_dir = TRUE;
+	fr_window_set_extract_default_dir (window, extract_to_dir, TRUE);
 
 	overwrite = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_overwrite_checkbutton));
 	skip_newer = !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton)) && gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_not_newer_checkbutton));
@@ -223,7 +216,8 @@ extract_cb (GtkWidget   *w,
 			g_free (password);
 			password = NULL;
 		}
-	} else
+	}
+	else
 		password = NULL;
 
 	eel_gconf_set_boolean (PREF_EXTRACT_VIEW_FOLDER, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->e_view_folder_checkbutton)));
@@ -232,13 +226,15 @@ extract_cb (GtkWidget   *w,
 
 	file_list = NULL;
 
-	if (selected_files)
-		file_list = window_get_file_list_selection (window, TRUE, NULL);
-
+	if (selected_files){
+		file_list = data->selected_files;
+		data->selected_files = NULL;       /* do not the list when destroying the dialog. */
+	} 
 	else if (pattern_files) {
 		const char *pattern;
+
 		pattern = gtk_entry_get_text (GTK_ENTRY (data->e_files_entry));
-		file_list = window_get_file_list_pattern (window, pattern);
+		file_list = fr_window_get_file_list_pattern (window, pattern);
 		if (file_list == NULL) {
 			gtk_widget_destroy (data->dialog);
 			g_free (extract_to_dir);
@@ -247,37 +243,34 @@ extract_cb (GtkWidget   *w,
 		}
 	}
 
+	if (selected_files) {
+		base_dir = data->base_dir_for_selection;
+		data->base_dir_for_selection = NULL;
+	}
+	else
+		base_dir = NULL;
+
 	/* close the dialog. */
 
 	gtk_widget_destroy (data->dialog);
 
 	/* extract ! */
 
-	if (eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE)) {
-		window->view_folder_after_extraction = TRUE;
-		g_free (window->folder_to_view);
-		window->folder_to_view = g_strdup (extract_to_dir);
-	}
+	fr_window_view_folder_after_extract (window, eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE));
+	fr_window_set_password (window, password);
 
-	if (password != NULL) {
-		g_free (window->password);
-		window->password = g_strdup (password);
-	}
-
-	if (selected_files)
-		base_dir = window_get_current_location (window);
-
-	window_archive_extract (window,
-				file_list,
-				extract_to_dir,
-				base_dir,
-				skip_newer,
-				overwrite,
-				junk_paths,
-				password);
+	fr_window_archive_extract (window,
+				   file_list,
+				   extract_to_dir,
+				   base_dir,
+				   skip_newer,
+				   overwrite,
+				   junk_paths,
+				   password);
 
 	path_list_free (file_list);
 	g_free (extract_to_dir);
+	g_free (base_dir);
 	g_free (password);
 
 	return TRUE;
@@ -464,17 +457,19 @@ create_extra_widget (DialogData *data)
 }
 
 
-void
-dlg_extract (GtkWidget *widget,
-	     gpointer   callback_data)
+static void
+dlg_extract__common (FrWindow *window,
+	             GList    *selected_files,
+	             char     *base_dir_for_selection)
 {
-	FRWindow   *window = callback_data;
 	DialogData *data;
 	GtkWidget  *file_sel;
 
 	data = g_new0 (DialogData, 1);
 
 	data->window = window;
+	data->selected_files = selected_files;
+	data->base_dir_for_selection = base_dir_for_selection;
 	data->extract_clicked = FALSE;
 
 	data->tooltips = gtk_tooltips_new ();
@@ -483,7 +478,7 @@ dlg_extract (GtkWidget *widget,
 
 	data->dialog = file_sel =
 		gtk_file_chooser_dialog_new (_("Extract"),
-					     GTK_WINDOW (data->window->app),
+					     GTK_WINDOW (data->window),
 					     GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					     FR_STOCK_EXTRACT, GTK_RESPONSE_OK,
@@ -493,21 +488,16 @@ dlg_extract (GtkWidget *widget,
 	gtk_window_set_default_size (GTK_WINDOW (file_sel), 530, 510);
 
 	gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (file_sel), FALSE);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), TRUE);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (file_sel), FALSE);
 	gtk_dialog_set_default_response (GTK_DIALOG (file_sel), GTK_RESPONSE_OK);
 
 	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (file_sel), create_extra_widget (data));
 
 	/* Set widgets data. */
 
-	if (window->extract_default_dir != NULL) {
-		const char *folder = window->extract_default_dir;
-		if (dir_is_temp_dir (folder))
-			folder = g_get_home_dir ();
-		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (file_sel), folder);
-	}
+	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (file_sel), fr_window_get_extract_default_dir (window));
 
-	if (_gtk_count_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (window->list_view))) > 0)
+	if (data->selected_files != NULL)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_selected_radiobutton), TRUE);
 	else {
 		gtk_widget_set_sensitive (data->e_selected_radiobutton, FALSE);
@@ -516,9 +506,9 @@ dlg_extract (GtkWidget *widget,
 
 	if (window->archive->command->propPassword) {
 		gtk_widget_set_sensitive (data->e_password_hbox, TRUE);
-		if (window->password != NULL)
-			_gtk_entry_set_locale_text (GTK_ENTRY (data->e_password_entry), window->password);
-	} else
+		_gtk_entry_set_locale_text (GTK_ENTRY (data->e_password_entry), fr_window_get_password (window));
+	}
+	else
 		gtk_widget_set_sensitive (data->e_password_hbox, FALSE);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->e_view_folder_checkbutton), eel_gconf_get_boolean (PREF_EXTRACT_VIEW_FOLDER, FALSE));
@@ -558,3 +548,29 @@ dlg_extract (GtkWidget *widget,
 	gtk_window_set_modal (GTK_WINDOW (file_sel),TRUE);
 	gtk_widget_show (file_sel);
 }
+
+
+void
+dlg_extract (GtkWidget *widget,
+	     gpointer   callback_data)
+{
+	FrWindow *window = callback_data;
+	GList    *files;
+	char     *base_dir;
+	
+	files = fr_window_get_selection (window, FALSE, &base_dir);
+	dlg_extract__common (window, files, base_dir);
+}
+
+
+void
+dlg_extract_folder_from_sidebar (GtkWidget *widget,
+	     			 gpointer   callback_data)
+{
+	FrWindow *window = callback_data;
+	GList    *files;
+	char     *base_dir;
+	
+	files = fr_window_get_selection (window, TRUE, &base_dir);
+	dlg_extract__common (window, files, base_dir);
+}
