@@ -53,6 +53,8 @@ mktime_from_string (char *month,
 	struct tm     tm = {0, };
 	char        **fields;
 
+	tm.tm_isdst = -1;
+
 	/* date */
 
 	if (month != NULL) {
@@ -79,7 +81,7 @@ mktime_from_string (char *month,
 
 		fields = g_strsplit (time_or_year, ":", 2);
 		if (fields[0] != NULL) {
-			tm.tm_hour = atoi (fields[0]) - 1;
+			tm.tm_hour = atoi (fields[0]);
 			if (fields[1] != NULL) 
 				tm.tm_min = atoi (fields[1]);
 		}
@@ -107,11 +109,16 @@ split_line_lha (char *line)
 		fields[i++] = g_strdup ("");
 		fields[i++] = g_strdup ("");
 		line += strlen ("[MS-DOS]");
-
-	} else if (strncmp (line, "[generic]", 9) == 0) {
+	} 
+	else if (strncmp (line, "[generic]", 9) == 0) {
 		fields[i++] = g_strdup ("");
 		fields[i++] = g_strdup ("");
 		line += strlen ("[generic]");
+	} 
+	else if (strncmp (line, "[unknown]", 9) == 0) {
+		fields[i++] = g_strdup ("");
+		fields[i++] = g_strdup ("");
+		line += strlen ("[unknown]");
 	}
 
 	scan = eat_spaces (line);
@@ -140,6 +147,9 @@ get_last_field_lha (char *line)
 	if (strncmp (line, "[generic]", 9) == 0) 
 		n--;
 
+	if (strncmp (line, "[unknown]", 9) == 0) 
+		n--;
+
 	field = eat_spaces (line);
 	for (i = 0; i < n; i++) {
 		field = strchr (field, ' ');
@@ -174,7 +184,7 @@ process_line (char     *line,
 
 	name_field = get_last_field_lha (line);
 
-	if (*name_field == '/') {
+	if (name_field && *name_field == '/') {
 		fdata->full_path = g_strdup (name_field);
 		fdata->original_path = fdata->full_path;
 	} else {
@@ -186,7 +196,7 @@ process_line (char     *line,
 
 	fdata->dir = line[0] == 'd';
 	if (fdata->dir)
-		fdata->name =  dir_name_from_path (fdata->full_path);
+		fdata->name = dir_name_from_path (fdata->full_path);
 	else
 		fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 
@@ -195,7 +205,7 @@ process_line (char     *line,
 	if (*fdata->name == 0)
 		file_data_free (fdata);
 	else
-		comm->file_list = g_list_prepend (comm->file_list, fdata);
+		fr_command_add_file (comm, fdata);
 }
 
 
