@@ -28,28 +28,28 @@
 #include "fr-process.h"
 
 #define FR_TYPE_COMMAND_ZIP            (fr_command_zip_get_type ())
-#define FR_COMMAND_ZIP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), FR_TYPE_COMMAND_ZIP, FRCommandZip))
-#define FR_COMMAND_ZIP_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), FR_TYPE_COMMAND_ZIP, FRCommandZipClass))
+#define FR_COMMAND_ZIP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), FR_TYPE_COMMAND_ZIP, FrCommandZip))
+#define FR_COMMAND_ZIP_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), FR_TYPE_COMMAND_ZIP, FrCommandZipClass))
 #define FR_IS_COMMAND_ZIP(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), FR_TYPE_COMMAND_ZIP))
 #define FR_IS_COMMAND_ZIP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), FR_TYPE_COMMAND_ZIP))
-#define FR_COMMAND_ZIP_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), FR_TYPE_COMMAND_ZIP, FRCommandZipClass))
+#define FR_COMMAND_ZIP_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), FR_TYPE_COMMAND_ZIP, FrCommandZipClass))
 
-typedef struct _FRCommandZip       FRCommandZip;
-typedef struct _FRCommandZipClass  FRCommandZipClass;
+typedef struct _FrCommandZip       FrCommandZip;
+typedef struct _FrCommandZipClass  FrCommandZipClass;
 
-struct _FRCommandZip
+struct _FrCommandZip
 {
-	FRCommand  __parent;
+	FrCommand  __parent;
 	gboolean   is_empty;
 };
 
-struct _FRCommandZipClass
+struct _FrCommandZipClass
 {
-	FRCommandClass __parent_class;
+	FrCommandClass __parent_class;
 };
 
 GType        fr_command_zip_get_type        (void);
-FRCommand*   fr_command_zip_new             (FRProcess *process,
+FrCommand*   fr_command_zip_new             (FrProcess *process,
 					     const char *filename);
 
 #endif /* FR_COMMAND_ZIP_H */
