@@ -35,7 +35,7 @@ static GtkActionEntry action_entries[] = {
 	{ "ViewMenu", NULL, N_("_View") },
 	{ "HelpMenu", NULL, N_("_Help") },
 	{ "ArrangeFilesMenu", NULL, N_("_Arrange Files") },
-	{ "OpenRecentMenu", NULL, N_("Open R_ecent") },
+	{ "OpenRecentMenu", NULL, N_("Open _Recent") },
 
 	{ "About", GTK_STOCK_ABOUT,
 	  NULL, NULL,
@@ -83,7 +83,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Rename the selection"),
 	  G_CALLBACK (activate_action_rename) },	   
 	{ "Delete", GTK_STOCK_DELETE,
-	  NULL, NULL,
+	  NULL, "Delete",
 	  N_("Delete the selection from the archive"),
 	  G_CALLBACK (activate_action_delete) },
 	  
@@ -113,7 +113,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Deselect all files"),
 	  G_CALLBACK (activate_action_deselect_all) },
 	{ "Extract", FR_STOCK_EXTRACT,
-	  N_("_Extract..."), NULL,
+	  N_("_Extract..."), "<control>E",
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract) },
 	{ "ExtractFolderFromSidebar", FR_STOCK_EXTRACT,
@@ -124,6 +124,11 @@ static GtkActionEntry action_entries[] = {
 	  N_("Extract"), NULL,
 	  N_("Extract files from the archive"),
 	  G_CALLBACK (activate_action_extract) },
+	{ "Find", GTK_STOCK_FIND,
+	  NULL, NULL,
+	  NULL,
+	  G_CALLBACK (activate_action_find) },
+	  
 	{ "LastOutput", NULL,
 	  N_("_Last Output"), NULL,
 	  N_("View the output produced by the last executed command"),
@@ -141,7 +146,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Open archive"),
 	  G_CALLBACK (activate_action_open) },
 	{ "OpenSelection", NULL,
-	  N_("Op_en With..."), NULL,
+	  N_("_Open With..."), NULL,
 	  N_("Open selected files with an application"),
 	  G_CALLBACK (activate_action_open_with) },
 	{ "Password", NULL,
@@ -149,7 +154,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Specify a password for this archive"),
 	  G_CALLBACK (activate_action_password) },
 	{ "Properties", GTK_STOCK_PROPERTIES,
-	  NULL, NULL,
+	  NULL, "<alt>Return",
 	  N_("Show archive properties"),
 	  G_CALLBACK (activate_action_properties) },
 	{ "Reload", GTK_STOCK_REFRESH,
@@ -161,7 +166,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Save the current archive with a different name"),
 	  G_CALLBACK (activate_action_save_as) },
 	{ "SelectAll", NULL,
-	  N_("Select _All"), "<control>A",
+	  N_("_Select All"), "<control>A",
 	  N_("Select all files"),
 	  G_CALLBACK (activate_action_select_all) },
 	{ "Stop", GTK_STOCK_STOP,
@@ -279,6 +284,8 @@ static const gchar *ui_info =
 "      <menuitem action='SelectAll'/>"
 "      <menuitem action='DeselectAll'/>"
 "      <separator/>"
+"      <menuitem action='Find'/>"
+"      <separator/>"
 "      <menuitem action='Password'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
